from dataclasses import dataclass, field
from dataclasses_json import dataclass_json
from sdk import utils


@dataclass_json
@dataclass
class EmbedAccessTokenResponse:
    r"""EmbedAccessTokenResponse
    An EmbedAccessTokenResponse contains a token that can be used to embed a Speakeasy dashboard.
    """
    
    access_token: str = field(metadata={'dataclasses_json': { 'letter_case': utils.field_name('access_token') }})
    



from dataclasses import dataclass, field
from typing import Optional
from sdk.models import shared
from sdk.models import shared


@dataclass
class GetAllAPIVersionsPathParams:
    api_id: str = field(metadata={'path_param': { 'field_name': 'apiID', 'style': 'simple', 'explode': False }})
    

@dataclass
class GetAllAPIVersionsOp:
    and_: bool = field(metadata={'query_param': { 'field_name': 'and' }})
    

@dataclass
class GetAllAPIVersionsQueryParams:
    metadata: Optional[dict[str, list[str]]] = field(default=None, metadata={'query_param': { 'field_name': 'metadata', 'style': 'deepObject', 'explode': True }})
    op: Optional[GetAllAPIVersionsOp] = field(default=None, metadata={'query_param': { 'field_name': 'op', 'style': 'deepObject', 'explode': True }})
    

@dataclass
class GetAllAPIVersionsRequest:
    path_params: GetAllAPIVersionsPathParams = field()
    query_params: GetAllAPIVersionsQueryParams = field()
    

@dataclass
class GetAllAPIVersionsResponse:
    content_type: str = field()
    status_code: int = field()
    apis: Optional[list[shared.API]] = field(default=None)
    error: Optional[shared.Error] = field(default=None)
    



from dataclasses import dataclass, field
from typing import Optional
from sdk.models import shared
from sdk.models import shared


@dataclass
class GetApisOp:
    and_: bool = field(metadata={'query_param': { 'field_name': 'and' }})
    

@dataclass
class GetApisQueryParams:
    metadata: Optional[dict[str, list[str]]] = field(default=None, metadata={'query_param': { 'field_name': 'metadata', 'style': 'deepObject', 'explode': True }})
    op: Optional[GetApisOp] = field(default=None, metadata={'query_param': { 'field_name': 'op', 'style': 'deepObject', 'explode': True }})
    

@dataclass
class GetApisRequest:
    query_params: GetApisQueryParams = field()
    

@dataclass
class GetApisResponse:
    content_type: str = field()
    status_code: int = field()
    apis: Optional[list[shared.API]] = field(default=None)
    error: Optional[shared.Error] = field(default=None)
    



from dataclasses import dataclass, field
from typing import Optional
from sdk.models import shared
from sdk.models import shared


@dataclass
class GetEmbedAccessTokenQueryParams:
    description: Optional[str] = field(default=None, metadata={'query_param': { 'field_name': 'description', 'style': 'form', 'explode': True }})
    duration: Optional[int] = field(default=None, metadata={'query_param': { 'field_name': 'duration', 'style': 'form', 'explode': True }})
    filters: Optional[shared.Filters] = field(default=None, metadata={'query_param': { 'field_name': 'filters', 'serialization': 'json' }})
    

@dataclass
class GetEmbedAccessTokenRequest:
    query_params: GetEmbedAccessTokenQueryParams = field()
    

@dataclass
class GetEmbedAccessTokenResponse:
    content_type: str = field()
    status_code: int = field()
    embed_access_token_response: Optional[shared.EmbedAccessTokenResponse] = field(default=None)
    error: Optional[shared.Error] = field(default=None)
    



from dataclasses import dataclass, field



@dataclass
class SchemeAPIKey:
    api_key: str = field(metadata={'security': { 'field_name': 'x-api-key' }})
    

@dataclass
class Security:
    api_key: SchemeAPIKey = field(metadata={'security': { 'scheme': True, 'type': 'apiKey', 'sub_type': 'header' }})
    



from dataclasses import dataclass, field
from typing import Optional
from sdk.models import shared
from sdk.models import shared


@dataclass
class GetValidEmbedAccessTokensResponse:
    content_type: str = field()
    status_code: int = field()
    embed_tokens: Optional[list[shared.EmbedToken]] = field(default=None)
    error: Optional[shared.Error] = field(default=None)
    



from dataclasses import dataclass, field
from typing import Optional
from sdk.models import shared
from sdk.models import shared


@dataclass
class RegisterSchemaPathParams:
    api_id: str = field(metadata={'path_param': { 'field_name': 'apiID', 'style': 'simple', 'explode': False }})
    version_id: str = field(metadata={'path_param': { 'field_name': 'versionID', 'style': 'simple', 'explode': False }})
    

@dataclass
class RegisterSchemaRequestBodyFile:
    content: bytes = field(metadata={'multipart_form': { 'content': True }})
    file: str = field(metadata={'multipart_form': { 'field_name': 'file' }})
    

@dataclass
class RegisterSchemaRequestBody:
    file: RegisterSchemaRequestBodyFile = field(metadata={'multipart_form': { 'file': True }})
    

@dataclass
class RegisterSchemaRequest:
    path_params: RegisterSchemaPathParams = field()
    request: RegisterSchemaRequestBody = field(metadata={'request': { 'media_type': 'multipart/form-data' }})
    

@dataclass
class RegisterSchemaResponse:
    content_type: str = field()
    status_code: int = field()
    error: Optional[shared.Error] = field(default=None)
    




__doc__ = """ SDK Documentation: https://docs.speakeasyapi.dev - The Speakeasy Platform Documentation"""
import requests
from sdk.models import shared
from . import utils

from .apiendpoints import APIEndpoints
from .apis import Apis
from .embeds import Embeds
from .metadata import Metadata
from .requests import Requests
from .schemas import Schemas


SERVER_PROD = "prod"

SERVERS = {
	SERVER_PROD: "https://api.prod.speakeasyapi.dev",
}


class SDK:
    r"""SDK Documentation: https://docs.speakeasyapi.dev - The Speakeasy Platform Documentation"""
    api_endpoints: APIEndpoints
    apis: Apis
    embeds: Embeds
    metadata: Metadata
    requests: Requests
    schemas: Schemas

    _client: requests.Session
    _security_client: requests.Session
    _security: shared.Security
    _server_url: str = SERVERS[SERVER_PROD]
    _language: str = "python"
    _sdk_version: str = "0.7.3"
    _gen_version: str = ""

    def __init__(self) -> None:
        self._client = requests.Session()
        self._security_client = requests.Session()
        self._init_sdks()


    def config_server_url(self, server_url: str, params: dict[str, str]):
        if params is not None:
            self._server_url = utils.replace_parameters(server_url, params)
        else:
            self._server_url = server_url

        self._init_sdks()
    
    def config_server(self, server: str, params: dict[str, str]):
        if not server in SERVERS:
            raise ValueError("Invalid server")
        self.config_server_url(SERVERS[server], params)
        self._init_sdks()
    

    def config_client(self, client: requests.Session):
        self._client = client
        
        if self._security is not None:
            self._security_client = utils.configure_security_client(self._client, self._security)
        self._init_sdks()
    

    def config_security(self, security: shared.Security):
        self._security = security
        self._security_client = utils.configure_security_client(self._client, security)
        self._init_sdks()
    
    
    def _init_sdks(self):
        
        self.api_endpoints = APIEndpoints(
            self._client,
            self._security_client,
            self._server_url,
            self._language,
            self._sdk_version,
            self._gen_version
        )
        
        self.apis = Apis(
            self._client,
            self._security_client,
            self._server_url,
            self._language,
            self._sdk_version,
            self._gen_version
        )
        
        self.embeds = Embeds(
            self._client,
            self._security_client,
            self._server_url,
            self._language,
            self._sdk_version,
            self._gen_version
        )
        
        self.metadata = Metadata(
            self._client,
            self._security_client,
            self._server_url,
            self._language,
            self._sdk_version,
            self._gen_version
        )
        
        self.requests = Requests(
            self._client,
            self._security_client,
            self._server_url,
            self._language,
            self._sdk_version,
            self._gen_version
        )
        
        self.schemas = Schemas(
            self._client,
            self._security_client,
            self._server_url,
            self._language,
            self._sdk_version,
            self._gen_version
        )
    
    


from dataclasses import dataclass, field
from typing import Optional
from sdk.models import shared
from sdk.models import shared


@dataclass
class RevokeEmbedAccessTokenPathParams:
    token_id: str = field(metadata={'path_param': { 'field_name': 'tokenID', 'style': 'simple', 'explode': False }})
    

@dataclass
class RevokeEmbedAccessTokenRequest:
    path_params: RevokeEmbedAccessTokenPathParams = field()
    

@dataclass
class RevokeEmbedAccessTokenResponse:
    content_type: str = field()
    status_code: int = field()
    error: Optional[shared.Error] = field(default=None)
    



import requests
from typing import Optional
from sdk.models import shared, operations
from . import utils

class Metadata:
    _client: requests.Session
    _security_client: requests.Session
    _server_url: str
    _language: str
    _sdk_version: str
    _gen_version: str

    def __init__(self, client: requests.Session, security_client: requests.Session, server_url: str, language: str, sdk_version: str, gen_version: str) -> None:
        self._client = client
        self._security_client = security_client
        self._server_url = server_url
        self._language = language
        self._sdk_version = sdk_version
        self._gen_version = gen_version

    
    def delete_version_metadata(self, request: operations.DeleteVersionMetadataRequest) -> operations.DeleteVersionMetadataResponse:
        r"""Delete metadata for a particular apiID and versionID.
        """
        
        base_url = self._server_url
        
        url = utils.generate_url(base_url, "/v1/apis/{apiID}/version/{versionID}/metadata/{metaKey}/{metaValue}", request.path_params)
        
        
        client = self._security_client
        
        r = client.request("DELETE", url)
        content_type = r.headers.get("Content-Type")

        res = operations.DeleteVersionMetadataResponse(status_code=r.status_code, content_type=content_type)
        
        if r.status_code == 200:
            pass
        else:
            if utils.match_content_type(content_type, "application/json"):
                out = utils.unmarshal_json(r.text, Optional[shared.Error])
                res.error = out

        return res

    
    def get_version_metadata(self, request: operations.GetVersionMetadataRequest) -> operations.GetVersionMetadataResponse:
        r"""Get all metadata for a particular apiID and versionID.
        """
        
        base_url = self._server_url
        
        url = utils.generate_url(base_url, "/v1/apis/{apiID}/version/{versionID}/metadata", request.path_params)
        
        
        client = self._security_client
        
        r = client.request("GET", url)
        content_type = r.headers.get("Content-Type")

        res = operations.GetVersionMetadataResponse(status_code=r.status_code, content_type=content_type)
        
        if r.status_code == 200:
            if utils.match_content_type(content_type, "application/json"):
                out = utils.unmarshal_json(r.text, Optional[list[shared.VersionMetadata]])
                res.version_metadata = out
        else:
            if utils.match_content_type(content_type, "application/json"):
                out = utils.unmarshal_json(r.text, Optional[shared.Error])
                res.error = out

        return res

    
    def insert_version_metadata(self, request: operations.InsertVersionMetadataRequest) -> operations.InsertVersionMetadataResponse:
        r"""Insert metadata for a particular apiID and versionID.
        """
        
        base_url = self._server_url
        
        url = utils.generate_url(base_url, "/v1/apis/{apiID}/version/{versionID}/metadata", request.path_params)
        
        headers = {}
        req_content_type, data, form = utils.serialize_request_body(request)
        if req_content_type != "multipart/form-data" and req_content_type != "multipart/mixed":
            headers["content-type"] = req_content_type
        if data is None and form is None:
           raise Exception('request body is required')
        
        client = self._security_client
        
        r = client.request("POST", url, data=data, files=form, headers=headers)
        content_type = r.headers.get("Content-Type")

        res = operations.InsertVersionMetadataResponse(status_code=r.status_code, content_type=content_type)
        
        if r.status_code == 200:
            if utils.match_content_type(content_type, "application/json"):
                out = utils.unmarshal_json(r.text, Optional[shared.VersionMetadata])
                res.version_metadata = out
        else:
            if utils.match_content_type(content_type, "application/json"):
                out = utils.unmarshal_json(r.text, Optional[shared.Error])
                res.error = out

        return res

    


from dataclasses import dataclass, field
from typing import Optional
from sdk.models import shared
from sdk.models import shared


@dataclass
class UpsertAPIPathParams:
    api_id: str = field(metadata={'path_param': { 'field_name': 'apiID', 'style': 'simple', 'explode': False }})
    

@dataclass
class UpsertAPIRequest:
    path_params: UpsertAPIPathParams = field()
    request: shared.APIInput = field(metadata={'request': { 'media_type': 'application/json' }})
    

@dataclass
class UpsertAPIResponse:
    content_type: str = field()
    status_code: int = field()
    api: Optional[shared.API] = field(default=None)
    error: Optional[shared.Error] = field(default=None)
    



from dataclasses import dataclass, field
from typing import Optional
from sdk.models import shared
from sdk.models import shared


@dataclass
class QueryEventLogQueryParams:
    filters: Optional[shared.Filters] = field(default=None, metadata={'query_param': { 'field_name': 'filters', 'serialization': 'json' }})
    

@dataclass
class QueryEventLogRequest:
    query_params: QueryEventLogQueryParams = field()
    

@dataclass
class QueryEventLogResponse:
    content_type: str = field()
    status_code: int = field()
    bounded_requests: Optional[list[shared.BoundedRequest]] = field(default=None)
    error: Optional[shared.Error] = field(default=None)
    



import base64
import cgi
import json
import re
from dataclasses import Field, dataclass, fields, is_dataclass, make_dataclass
from datetime import date, datetime
from typing import Callable, Tuple, Union, get_args, get_origin
from xmlrpc.client import boolean

import requests
from dataclasses_json import DataClassJsonMixin


class SecurityClient:
    client: requests.Session
    query_params: dict[str, str] = {}

    def __init__(self, client: requests.Session):
        self.client = client

    def request(self, method, url, **kwargs):
        params = kwargs.get('params', {})
        kwargs["params"] = self.query_params | params

        return self.client.request(method, url, **kwargs)


def configure_security_client(client: requests.Session, security: dataclass):
    client = SecurityClient(client)

    sec_fields: Tuple[Field, ...] = fields(security)
    for sec_field in sec_fields:
        value = getattr(security, sec_field.name)
        if value is None:
            continue

        metadata = sec_field.metadata.get('security')
        if metadata is None:
            continue
        if metadata.get('option'):
            _parse_security_option(client, value)
            return
        elif metadata.get('scheme'):
            _parse_security_scheme(client, metadata, value)

    return client


def _parse_security_option(client: SecurityClient, option: dataclass):
    opt_fields: Tuple[Field, ...] = fields(option)
    for opt_field in opt_fields:
        metadata = opt_field.metadata.get('security')
        if metadata is None or metadata.get('scheme') is None:
            continue
        _parse_security_scheme(client, metadata.get(
            'scheme'), getattr(option, opt_field.name))


def _parse_security_scheme(client: SecurityClient, scheme_metadata: dict, scheme: dataclass):
    scheme_type = scheme_metadata.get('type')
    sub_type = scheme_metadata.get('sub_type')

    if scheme_type == 'http' and sub_type == 'basic':
        _parse_basic_auth_scheme(client, scheme)
        return

    scheme_fields: Tuple[Field, ...] = fields(scheme)
    for scheme_field in scheme_fields:
        metadata = scheme_field.metadata.get('security')
        if metadata is None or metadata.get('field_name') is None:
            continue

        header_name = metadata.get('field_name')
        value = getattr(scheme, scheme_field.name)

        if scheme_type == "apiKey":
            if sub_type == 'header':
                client.client.headers[header_name] = value
            elif sub_type == 'query':
                client.query_params[header_name] = value
            elif sub_type == 'cookie':
                client.client.cookies[header_name] = value
            else:
                raise Exception('not supported')
        elif scheme_type == "openIdConnect":
            client.client.headers[header_name] = value
        elif scheme_type == 'oauth2':
            client.client.headers[header_name] = value
        elif scheme_type == 'http':
            if sub_type == 'bearer':
                client.client.headers[header_name] = value
            else:
                raise Exception('not supported')
        else:
            raise Exception('not supported')


def _parse_basic_auth_scheme(client: SecurityClient, scheme: dataclass):
    username, password = ""

    scheme_fields: Tuple[Field, ...] = fields(scheme)
    for scheme_field in scheme_fields:
        metadata = scheme_field.metadata.get('security')
        if metadata is None or metadata.get('field_name') is None:
            continue

        field_name = metadata.get('field_name')
        value = getattr(scheme, scheme_field.name)

        if field_name == 'username':
            username = value
        if field_name == 'password':
            password = value

    data = f'{username}:{password}'.encode()
    client.client.headers['Authorization'] = f'Basic {base64.b64encode(data)}'


def generate_url(server_url: str, path: str, path_params: dataclass) -> str:
    param_fields: Tuple[Field, ...] = fields(path_params)
    for f in param_fields:
        param_metadata = f.metadata.get('path_param')
        if not param_metadata:
            continue
        if param_metadata.get('style', 'simple') == 'simple':
            param = getattr(path_params, f.name)
            if type(param) is list:
                pp_vals: list[str] = []
                for pp_val in param:
                    pp_vals.append(str(pp_val))
                path = path.replace(
                    '{' + param_metadata.get('field_name', f.name) + '}', ",".join(pp_vals), 1)
            elif type(param) is map:
                pp_vals: list[str] = []
                for pp_key in param:
                    if param_metadata.get('explode'):
                        pp_vals.append(f"{pp_key}={param[pp_key]}")
                    else:
                        pp_vals.append(f"{pp_key},{param[pp_key]}")
                path = path.replace(
                    '{' + param_metadata.get('field_name', f.name) + '}', ",".join(pp_vals), 1)
            elif not isinstance(param, (str, int, float, complex, bool)):
                pp_vals: list[str] = []
                attrs: list[str] = [p for p in dir(param) if not p.startswith(
                    '__') and not callable(getattr(param, p))]
                for attr in attrs:
                    field: Field = _get_field_from_attr(param, attr)

                    param_value_metadata = field.metadata.get('path_param')
                    if not param_value_metadata:
                        continue

                    parm_name = param_value_metadata.get('field_name', f.name)

                    param_field_val = getattr(param, attr)
                    if field is not None and is_optional(field) and param_field_val is None:
                        continue
                    elif param_metadata.get('explode'):
                        pp_vals.append(f"{parm_name}={param_field_val}")
                    else:
                        pp_vals.append(f"{parm_name},{param_field_val}")
                path = path.replace(
                    '{' + param_metadata.get('field_name', f.name) + '}', ",".join(pp_vals), 1)
            else:
                path = path.replace(
                    '{' + param_metadata.get('field_name', f.name) + '}', str(param), 1)

    return server_url.removesuffix("/") + path


def _get_field_from_attr(obj, attr: str) -> Field:
    pp_fields: Tuple[Field, ...] = fields(obj)
    for pp_field in pp_fields:
        if pp_field.name == attr:
            return pp_field
    return None


def is_optional(field):
    return get_origin(field) is Union and type(None) in get_args(field)


def replace_parameters(string_with_params: str, params: dict[str, str]) -> str:
    for key, value in params.items():
        string_with_params = string_with_params.replace(
            '{' + key + '}', value)

    return string_with_params


def get_query_params(query_params: dataclass) -> dict[str, list[str]]:
    if query_params is None:
        return {}

    params: dict[str, list[str]] = {}

    param_fields: Tuple[Field, ...] = fields(query_params)
    for f in param_fields:
        metadata = f.metadata.get('query_param')
        if not metadata:
            continue

        serialization = metadata.get('serialization', '')
        if serialization != '':
            params = params | _get_serialized_query_params(
                metadata, f.name, getattr(query_params, f.name))
        else:
            style = metadata.get('style', 'form')
            if style == 'deepObject':
                params = params | _get_deep_object_query_params(
                    metadata, f.name, getattr(query_params, f.name))
            elif style == 'form':
                params = params | _get_form_query_params(
                    metadata, f.name, getattr(query_params, f.name))
            else:
                raise Exception('not yet implemented')

    return params


def get_headers(headers_params: dataclass) -> dict[str, str]:
    if headers_params is None:
        return {}

    headers: dict[str, str] = {}

    param_fields: Tuple[Field, ...] = fields(headers_params)
    for f in param_fields:
        metadata = f.metadata.get('header_param')
        if not metadata:
            continue

        value = _serialize_header(metadata.get(
            'explode', False), getattr(headers_params, f.name))
        if value != '':
            headers[metadata.get('field_name', f.name)] = value

    return headers


def _get_serialized_query_params(metadata: dict, field_name: str, obj: any) -> dict[str, list[str]]:
    params: dict[str, list[str]] = {}

    serialization = metadata.get('serialization', '')

    if serialization == 'json':
        params[metadata.get("field_name", field_name)] = marshal_json(obj)

    return params


def _get_deep_object_query_params(metadata: dict, field_name: str, obj: any) -> dict[str, list[str]]:
    params: dict[str, list[str]] = {}

    if is_dataclass(obj):
        obj_fields: Tuple[Field, ...] = fields(obj)
        for obj_field in obj_fields:
            obj_param_metadata = obj_field.metadata.get('query_param')
            if not obj_param_metadata:
                continue
            params[
                f'{metadata.get("field_name", field_name)}[{obj_param_metadata.get("field_name", obj_field.name)}]'] = [
                getattr(obj, obj_field.name)]
    elif isinstance(obj, dict):
        for key, value in obj.items():
            if isinstance(value, list):
                params[f'{metadata.get("field_name", field_name)}[{key}]'] = value
            else:
                params[f'{metadata.get("field_name", field_name)}[{key}]'] = [
                    value]
    return params


def _get_query_param_field_name(obj_field: Field) -> str:
    obj_param_metadata = obj_field.metadata.get('query_param')

    if not obj_param_metadata:
        return ""

    return obj_param_metadata.get("field_name", obj_field.name)


def _get_form_query_params(metadata: dict, field_name: str, obj: any) -> dict[str, list[str]]:
    return _populate_form(field_name, metadata.get("explode", True), obj, _get_query_param_field_name)


def serialize_request_body(request: dataclass) -> Tuple[str, any, any]:
    if request is None:
        return None, None, None

    request_val = getattr(request, "request")
    if request_val is None:
        raise Exception("request body not found")

    request_fields: Tuple[Field, ...] = fields(request)
    request_metadata = None

    for f in request_fields:
        if f.name == "request":
            request_metadata = f.metadata.get('request')
            break

    if request_metadata is not None:
        # single request
        return serialize_content_type("request", request_metadata, request_val)

    request_fields: Tuple[Field, ...] = fields(request_val)
    for f in request_fields:
        req = getattr(request_val, f.name)
        if req is None:
            continue

        request_metadata = f.metadata.get('request')
        if request_metadata is None:
            raise Exception(
                f'missing request tag on request body field {f.name}')

        return serialize_content_type(f.name, request_metadata, req)


def serialize_content_type(request_field_name: str, metadata, request: dataclass) -> Tuple[str, any, list[list[any]]]:
    media_type = metadata.get('media_type', 'application/octet-stream')

    if re.match(r'(application|text)\/.*?\+*json.*', media_type) != None:
        return media_type, marshal_json(request).encode(), None
    elif re.match(r'`multipart\/.*', media_type) != None:
        return serialize_multipart_form(media_type, request)
    elif re.match(r'application\/x-www-form-urlencoded.*', media_type) != None:
        return media_type, serialize_form(request_field_name, request), None
    else:
        if isinstance(request, (bytes, bytearray)):
            return media_type, request, None
        if isinstance(request, str):
            return media_type, request, None
        else:
            raise Exception(
                f"invalid request body type {type(request)} for mediaType {metadata['media_type']}")


def serialize_multipart_form(media_type: str, request: dataclass) -> Tuple[str, any, list[list[any]]]:
    form: list[list[any]] = []
    request_fields: Tuple[Field, ...] = fields(request)
    for f in request_fields:
        field_metadata = f.metadata.get('multipart_form')
        if field_metadata is None:
            continue
        if field_metadata.get("file") is True:
            file = getattr(request, f.name)
            file_fields = fields(file)

            file_name = ""
            field_name = ""
            content = bytes()

            for file_field in file_fields:
                file_metadata = file_field.metadata.get('multipart_form')
                if file_metadata is None:
                    continue
                if file_metadata.get("content") is True:
                    content = getattr(file, file_field.name)
                else:
                    field_name = file_metadata.get(
                        "field_name", file_field.name)
                    file_name = getattr(file, file_field.name)
            if field_name == "" or file_name == "" or content == bytes():
                raise Exception('invalid multipart/form-data file')

            form.append([field_name, [file_name, content]])
        elif field_metadata.get("json") is True:
            form.append([field_metadata.get("field_name", f.name), [
                None, marshal_json(getattr(request, f.name)), "application/json"]])
        else:
            val = getattr(request, f.name)
            field_name = field_metadata.get("field_name", f.name)

            if isinstance(val, list):
                for value in val:
                    form.append([field_name + "[]", [None, value]])
            else:
                form.append([field_name, [None, val]])
    return media_type, None, form


def _get_form_field_name(obj_field: Field) -> str:
    obj_param_metadata = obj_field.metadata.get('form')

    if not obj_param_metadata:
        return ""

    return obj_param_metadata.get("field_name", obj_field.name)


def serialize_form(request_field_name: str, request: any) -> dict[str, any]:
    form: dict[str, list[str]] = {}

    if is_dataclass(request):
        request_fields: Tuple[Field, ...] = fields(request)
        for f in request_fields:
            field_metadata = f.metadata.get('form')

            if field_metadata is None:
                continue

            if field_metadata.get("json") is True:
                form[field_metadata.get("field_name", f.name)] = [
                    marshal_json(getattr(request, f.name))]
            else:
                if field_metadata.get("style", "form") == "form":
                    values = _populate_form(field_metadata.get("field_name", f.name), field_metadata.get(
                        "explode", True), getattr(request, f.name), _get_form_field_name)

                    for key, value in values.items():
                        for v in value:
                            if not key in form:
                                form[key] = []

                            form[key].append(v)
    elif isinstance(request, dict):
        for key, value in request.items():
            if isinstance(value, list):
                for v in value:
                    if not key in form:
                        form[key] = []
                    form[key].append(v)
            else:
                if not key in form:
                    form[key] = []
                form[key].append(value)
    elif isinstance(request, list):
        for value in request:
            if isinstance(value, list):
                for v in value:
                    if not key in form:
                        form[request_field_name] = []
                    form[request_field_name].append(v)
            else:
                if not key in form:
                    form[request_field_name] = []
                form[request_field_name].append(value)

    return form


def _populate_form(field_name: str, explode: boolean, obj: any, get_field_name_func: Callable) -> dict[str, list[str]]:
    params: dict[str, list[str]] = {}

    if is_dataclass(obj):
        items = []

        obj_fields: Tuple[Field, ...] = fields(obj)
        for obj_field in obj_fields:
            obj_field_name = get_field_name_func(obj_field)
            if obj_field_name == "":
                continue

            if explode:
                params[obj_field_name] = [getattr(obj, obj_field.name)]
            else:
                items.append(
                    f'{obj_field_name},{getattr(obj, obj_field.name)}')

        if len(items) > 0:
            params[field_name] = [','.join(items)]
    elif isinstance(obj, dict):
        items = []

        for key, value in obj.items():
            if explode:
                params[key] = value
            else:
                items.append(f'{key},{value}')

        if len(items) > 0:
            params[field_name] = [','.join(items)]
    elif isinstance(obj, list):
        items = []

        for value in obj:
            if explode:
                if not field_name in params:
                    params[field_name] = []
                params[field_name].append(value)
            else:
                items.append(value)

        if len(items) > 0:
            params[field_name] = [','.join(items)]
    else:
        params[field_name] = obj

    return params


def _serialize_header(explode: boolean, obj: any) -> str:
    if is_dataclass(obj):
        items = []

        obj_fields: Tuple[Field, ...] = fields(obj)
        for obj_field in obj_fields:
            obj_param_metadata = obj_field.metadata.get('header_param')

            if not obj_param_metadata:
                continue

            obj_field_name = obj_param_metadata.get(
                "field_name", obj_field.name)
            if obj_field_name == "":
                continue

            if explode:
                items.append(
                    f'{obj_field_name}={getattr(obj, obj_field.name)}')
            else:
                items.append(obj_field_name)
                items.append(getattr(obj, obj_field.name))

        if len(items) > 0:
            return [','.join(items)]
    elif isinstance(obj, dict):
        items = []

        for key, value in obj.items():
            if explode:
                items.append(f'{key}={value}')
            else:
                items.append(key)
                items.append(value)

        if len(items) > 0:
            return [','.join(items)]
    elif isinstance(obj, list):
        return [','.join(obj)]
    else:
        return f'{obj}'


def unmarshal_json(data, t):
    Unmarhsal = make_dataclass('Unmarhsal', [('res', t)],
                               bases=(DataClassJsonMixin,))
    d = json.loads(data)
    out = Unmarhsal.from_dict({"res": d})
    return out.res


def marshal_json(c):
    Marshal = make_dataclass('Marshal', [('res', type(c))],
                             bases=(DataClassJsonMixin,))
    m = Marshal(res=c)
    d = m.to_dict()
    return json.dumps(d["res"])


def match_content_type(content_type: str, pattern: str) -> boolean:
    if content_type == pattern or pattern == "*" or pattern == "*/*":
        return True

    media_type, _ = cgi.parse_header(content_type)

    if media_type == pattern:
        return True

    parts = media_type.split("/")
    if len(parts) == 2:
        if f'{parts[0]}/*' == pattern or f'*/{parts[1]}' == pattern:
            return True

    return False


def datetimeisoformat(optional: bool):
    def isoformatoptional(v):
        if optional and v is None:
            return None
        return datetime.isoformat(v)

    return isoformatoptional


def dateisoformat(optional: bool):
    def isoformatoptional(v):
        if optional and v is None:
            return None
        return date.isoformat(v)

    return isoformatoptional


def field_name(name):
    def override(_, _field_name=name):
        return _field_name
    return override



from dataclasses import dataclass, field
from dataclasses_json import dataclass_json
from sdk import utils


@dataclass_json
@dataclass
class SchemaDiffValueChange:
    from_: str = field(metadata={'dataclasses_json': { 'letter_case': utils.field_name('From') }})
    to: str = field(metadata={'dataclasses_json': { 'letter_case': utils.field_name('To') }})
    

@dataclass_json
@dataclass
class SchemaDiff:
    r"""SchemaDiff
    A SchemaDiff represents a diff of two Schemas.
    """
    
    additions: list[str] = field(metadata={'dataclasses_json': { 'letter_case': utils.field_name('additions') }})
    deletions: list[str] = field(metadata={'dataclasses_json': { 'letter_case': utils.field_name('deletions') }})
    modifications: dict[str, SchemaDiffValueChange] = field(metadata={'dataclasses_json': { 'letter_case': utils.field_name('modifications') }})
    



from dataclasses import dataclass, field
from typing import Optional
from sdk.models import shared
from sdk.models import shared


@dataclass
class GetAPIEndpointPathParams:
    api_endpoint_id: str = field(metadata={'path_param': { 'field_name': 'apiEndpointID', 'style': 'simple', 'explode': False }})
    api_id: str = field(metadata={'path_param': { 'field_name': 'apiID', 'style': 'simple', 'explode': False }})
    version_id: str = field(metadata={'path_param': { 'field_name': 'versionID', 'style': 'simple', 'explode': False }})
    

@dataclass
class GetAPIEndpointRequest:
    path_params: GetAPIEndpointPathParams = field()
    

@dataclass
class GetAPIEndpointResponse:
    content_type: str = field()
    status_code: int = field()
    api_endpoint: Optional[shared.APIEndpoint] = field(default=None)
    error: Optional[shared.Error] = field(default=None)
    



from dataclasses import dataclass, field
from typing import Optional
from sdk.models import shared
from sdk.models import shared


@dataclass
class GetSchemaRevisionPathParams:
    api_id: str = field(metadata={'path_param': { 'field_name': 'apiID', 'style': 'simple', 'explode': False }})
    revision_id: str = field(metadata={'path_param': { 'field_name': 'revisionID', 'style': 'simple', 'explode': False }})
    version_id: str = field(metadata={'path_param': { 'field_name': 'versionID', 'style': 'simple', 'explode': False }})
    

@dataclass
class GetSchemaRevisionRequest:
    path_params: GetSchemaRevisionPathParams = field()
    

@dataclass
class GetSchemaRevisionResponse:
    content_type: str = field()
    status_code: int = field()
    error: Optional[shared.Error] = field(default=None)
    schema: Optional[shared.Schema] = field(default=None)
    
